import edu.wsu.KheperaSimulator.*;
import java.util.ArrayList;
public class Channel implements Runnable{
	
	public static int NUMBER_OF_SENSORS = 8; // number of RobotController sensors
	public static int STORE_COUNT = 5; // Number of sensor input values to be stored
	
	private RobotController rc;
	private ArrayList[] lightValues;
	private ArrayList[] distanceValues;
	boolean finished;
	
	/**
	 * Create a new Channel, store sensor values in lists.
	 * 
	 * @param rc The RobotController used for input values
	 */
	public Channel(RobotController rc){
		this.rc = rc;
		lightValues = new ArrayList[NUMBER_OF_SENSORS];
		for(int i=0;i<NUMBER_OF_SENSORS;i++){
			lightValues[i] = new ArrayList(); // One list pr. sensor.
		}
		distanceValues = new ArrayList[NUMBER_OF_SENSORS];
		for(int i=0;i<NUMBER_OF_SENSORS;i++){
			distanceValues[i] = new ArrayList(); // One list pr. sensor.
		}
		finished = false;
	}
	
	/**
	 * Gets the validated mean of 5 sensor values
	 * 
	 * @param id SensorID
	 * @return mean value of the stored light values
	 */
	public int getLightValue(int id){
		return getLightMeanValue(id);
	}
	
	public int getDistanceValue(int id){
		/* not implemented */
		int returnvalue = 0;
		int count = 0;
		int mean = 0;
		for(int i=0; i<distanceValues[id].size(); i++) {
			mean += ((Integer)distanceValues[id].get(i)).intValue();
			count++;
		}
		if(count>0){
			System.out.println(mean);
			returnvalue=mean / count;
		}
		return returnvalue;
	}
	
	/**
	 * run() - the main loop of the Channel class. Update the list of all sensors.
	 */
	public void run(){
		while(!finished){
			for(int id=0;id<NUMBER_OF_SENSORS;id++){
				updateLightValue(id); // update all light sensors
				updateDistanceValue(id);
			}
			try{
				Thread.sleep(5);
			}catch (InterruptedException e){}
			
		}
	}
	
	/**
	 * Updates light value of a sensor and validates the input.
	 * 
	 * @param id sensorID
	 */
	private void updateLightValue(int id){
		if(id >= 0 && id < NUMBER_OF_SENSORS){
			int value = rc.getLightValue(id); // get the input value from the RobotController
			if( value >= 0 && value <= 500){ // validate the input
				lightValues[id].add(0, new Integer(value));
				/* Keep the list the size of STORE_COUNT */
				while(lightValues[id].size() > STORE_COUNT){
					lightValues[id].remove(STORE_COUNT);
				}
			}
		}
	}
	private void updateDistanceValue(int id){
		if(id >= 0 && id < NUMBER_OF_SENSORS){
			int value = rc.getDistanceValue(id); // get the input value from the RobotController
			if( value >= 0 && value <= 1024){ // validate the input
				distanceValues[id].add(0, new Integer(value));
				/* Keep the list the size of STORE_COUNT */
				while(distanceValues[id].size() > STORE_COUNT){
					distanceValues[id].remove(STORE_COUNT);
				}
			}
		}
	}
	
	/**
	 * Get the mean value of the stored values of a light sensor.
	 * 
	 * @param id sensorID
	 * @return mean value
	 */
	private int getLightMeanValue(int id){
		int value = 0;
		int count = 0;
		for(int i=0;i<lightValues[id].size();i++){
			value += ((Integer)lightValues[id].get(i)).intValue();
			count++;
		}
		if(count > 0){
			return value/count; 
		}else{
			return 0;
		}
	}
	
	
	/**
	 * Stop the Channel processing.
	 *
	 */
	public void finish(){
		finished = true;
	}
}
