import edu.wsu.KheperaSimulator.RobotController;
import edu.wsu.KheperaSimulator.KSGripperStates;

public class MazeSolver extends RobotController {

	public final int MIN_WALL_DISTANCE = 100;
	public final int MAX_LIGHT_VALUE = 350;
	public int stepNumber = 0;
	private Channel ch;

	public MazeSolver(){
		/**
	     * This controller "polls" (I.E. manually checks) all of its sensors and then
		 * makes an action choice based on what it sees.  The wait time is the amount of time
		 * that elapses between every cycle of check/execute.  It is measured in miliseconds and
		 * is set by changing the number in the setWaitTime(number) command below.  Currently it is
		 * set to 5... which means a sensor check/action decision is done every 5 miliseconds.  You
		 * can change this if you want... but if you make it too long... the robot could crash into
		 * something while the controller is waiting to do its next sensor check.  Remember, unless you
		 * stop them... the motors will keep on doing what you tell them.
	     */
		setWaitTime(5);
		ch = new Channel(this);
		new Thread(ch).start();
	}

	/**
	 * This method will be called after this controller has been started
	 * and then again every so often until you stop the controller.  Specifically, it will be called
	 * once every how ever many miliseconds you specified in the call to setWaitTime()
	 */
	public void doWork() throws Exception {
		stepNumber++;
		if (stepNumber == 1) moveForward();
		else if (stepNumber == 50) turnRight();
		else if (stepNumber == 55) moveForward();
		else if (stepNumber == 480) turnLeft();
		else if (stepNumber == 500) moveForward();
		else if (stepNumber == 700) turnRight();
		else if (stepNumber == 720) moveForward();
		else if (stepNumber == 800) turnRight();
		else if (stepNumber == 810) moveForward();
		else if (stepNumber == 820) setGripperState(KSGripperStates.GRIP_OPEN);
		else if (stepNumber == 830) setArmState(KSGripperStates.ARM_DOWN);
		else if (stepNumber == 1200) stop();
		else if (stepNumber == 1210) setGripperState(KSGripperStates.GRIP_CLOSED);
		else if (stepNumber == 1220) setArmState(KSGripperStates.ARM_UP);
		else if (stepNumber == 1250) turnRight();
		else if (stepNumber == 1300) moveForward();
		else if (stepNumber == 1600) turnLeft();
		else if (stepNumber == 1624) moveForward();
		else if (stepNumber == 1870) turnRight();
		else if (stepNumber == 1890) moveForward();
		else if (stepNumber == 2100) turnLeft();
		else if (stepNumber == 2110) moveForward();
		else if (stepNumber == 2500) stop();
	}

	public void close() throws Exception{}

	public void moveForward(){
		/* if(ch.getDistanceValue(3) >= MIN_WALL_DISTANCE  || ch.getDistanceValue(2) >= MIN_WALL_DISTANCE
		  || ch.getLightValue(3) <= MAX_LIGHT_VALUE || ch.getLightValue(2) <= MAX_LIGHT_VALUE)
		{
			turnRight();
		}
		else if (ch.getDistanceValue(4) >= MIN_WALL_DISTANCE || ch.getDistanceValue(5) >= MIN_WALL_DISTANCE
		  || ch.getLightValue(4) <= MAX_LIGHT_VALUE || ch.getLightValue(5) <= MAX_LIGHT_VALUE)
		{
			turnLeft();
		}
		else
		{
			goStraightOrTurn();
		} */
		goStraight();
	}

	public void goStraightOrTurn()
	{
		int random = (int) (Math.random() * 16);
		if (random == 0) turnLeft();
		else if (random == 1) turnRight();
		else goStraight();
	}

	public void stop(){
		setMotorSpeeds(0,0);
	}

	/**
	 * This method will make the robot turn right.
	 */
	public void turnRight(){
		setMotorSpeeds(3, -3);
	}

	/**
	 * This method will make the robot turn left.
	 */
	public void turnLeft(){
		setMotorSpeeds(-3,3);
	}

	/**
	 * This method will make the robot go straight ahead.
	 */
	public void goStraight(){
		setMotorSpeeds(2,2);
	}

	public void backUp()
	{
		setMotorSpeeds(-2, -2);
	}

	public void pickUpBall(){
		setGripperState(KSGripperStates.GRIP_OPEN); // Open gripper
		setArmState(KSGripperStates.ARM_DOWN); // Lower arms;
		sleep(100);		// wait 100ms
		setGripperState(KSGripperStates.GRIP_CLOSED); // Close gripper
		sleep(100);		// wait 100ms
		setArmState(KSGripperStates.ARM_UP); // lift arms;
		sleep(100);		// wait 100ms
	}

	protected void setFinished(boolean finished){
		super.setFinished(finished);
	}

}
