import edu.wsu.KheperaSimulator.RobotController;
import edu.wsu.KheperaSimulator.KSGripperStates;

public class WallFollower extends RobotController {

	public final int MIN_WALL_DISTANCE = 100;
	private Channel ch;

	public WallFollower(){
		/**
	     * This controller "polls" (I.E. manually checks) all of its sensors and then
		 * makes an action choice based on what it sees.  The wait time is the amount of time
		 * that elapses between every cycle of check/execute.  It is measured in miliseconds and
		 * is set by changing the number in the setWaitTime(number) command below.  Currently it is
		 * set to 5... which means a sensor check/action decision is done every 5 miliseconds.  You
		 * can change this if you want... but if you make it too long... the robot could crash into
		 * something while the controller is waiting to do its next sensor check.  Remember, unless you
		 * stop them... the motors will keep on doing what you tell them.
	     */
		setWaitTime(1);
		ch = new Channel(this);
		new Thread(ch).start();
	}

	/**
	 * This method will be called after this controller has been started
	 * and then again every so often until you stop the controller.  Specifically, it will be called
	 * once every how ever many miliseconds you specified in the call to setWaitTime()
	 */
	public void doWork() throws Exception {
		if (!ch.getState())
		{
			backUp();
		}
		else
		{
			moveForward();
		}
	}

	public void close() throws Exception{}

	public void moveForward(){
		if(ch.getDistanceValue(3) >= MIN_WALL_DISTANCE  || ch.getDistanceValue(2) >= MIN_WALL_DISTANCE)
		{
			turnRight();
		}
		else
		{
			goStraight();
		}

	}

	public void stop(){
		setMotorSpeeds(0,0);
	}

	/**
	 * This method will make the robot turn right.
	 */
	public void turnRight(){
		setMotorSpeeds(3, -3);
	}

	/**
	 * This method will make the robot turn left.
	 */
	public void turnLeft(){
		setMotorSpeeds(-2,2);
	}

	/**
	 * This method will make the robot go straight ahead.
	 */
	public void goStraight(){
		setMotorSpeeds(2,2);
	}

	public void backUp()
	{
		setMotorSpeeds(-2, -2);
	}

	protected void setFinished(boolean finished){
		super.setFinished(finished);
	}

}
